package com.jsonvsxml_android.app;

import android.app.Activity;
import android.content.res.AssetManager;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import org.json.JSONArray;
import org.json.JSONObject;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;

public class MainActivity extends Activity
{
    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        try
        {
            for (int i = 0; i < 10; i++)
            {
                String executionString = String.format("Test Run %d: ", i + 1);

                long startTime = System.currentTimeMillis();
                ArrayList<String> employeesXML = getAllEmployeesXML();
                long executionTime = System.currentTimeMillis() - startTime;
                executionString = String.format("%s XML time %.6f seconds", executionString, executionTime / 1000.0);

                startTime = System.currentTimeMillis();
                ArrayList<String> employeesJSON = getAllEmployeesJSON();
                executionTime = System.currentTimeMillis() - startTime;
                Log.i("TB", String.format("%s JSON time %.6f seconds", executionString, executionTime / 1000.0));
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings)
        {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    private String getAssetFileContent(String fileName) throws Exception
    {
        String contentString = null;
        AssetManager am = getAssets();
        try
        {
            InputStream is = am.open(fileName);
            int length = is.available();
            byte[] data = new byte[length];
            is.read(data);
            contentString = new String(data);
        }
        catch (Exception e1)
        {
            e1.printStackTrace();
        }
        return contentString;
    }

    private InputStream getAssetInputStream(String fileName) throws Exception
    {
        AssetManager am = getAssets();
        try
        {
            InputStream is = am.open(fileName);
            return is;
        }
        catch (Exception e1)
        {
            e1.printStackTrace();
        }

        return null;
    }

    private ArrayList<String> getAllEmployeesXML() throws Exception
    {
        ArrayList<String> employees = new ArrayList<String>();

        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = dbf.newDocumentBuilder();
        Document doc = db.parse(new InputSource(getAssetInputStream("SouthWind.xml")));
        NodeList employeesNodes = doc.getElementsByTagName("Employee");
        for (int i = 0; i < employeesNodes.getLength(); i++)
        {
            Element employee = (Element) employeesNodes.item(i);
            String lastName = employee.getElementsByTagName("LastName").item(0).getTextContent();
            String firstName = employee.getElementsByTagName("FirstName").item(0).getTextContent();
            employees.add(String.format("%s %s", firstName, lastName));
        }

        return employees;
    }

    private ArrayList<String> getAllEmployeesJSON() throws Exception
    {
        ArrayList<String> employees = new ArrayList<String>();

        BufferedReader streamReader = new BufferedReader(new InputStreamReader(getAssetInputStream("SouthWind.json"), "UTF-8"));
        StringBuilder responseStrBuilder = new StringBuilder();

        String inputStr;
        while ((inputStr = streamReader.readLine()) != null)
            responseStrBuilder.append(inputStr);
        JSONObject jsonObject = new JSONObject(responseStrBuilder.toString());
        JSONObject rootObject = jsonObject.getJSONObject("windward-studios");
        JSONObject employeesObject = rootObject.getJSONObject("Employees");
        JSONArray employeesArray = employeesObject.getJSONArray("Employee");
        for (int i = 0; i < employeesArray.length(); i++)
        {
            JSONObject employeeObject = employeesArray.getJSONObject(i);
            String lastName = employeeObject.getString("LastName");
            String firstName = employeeObject.getString("FirstName");
            employees.add(String.format("%s %s", firstName, lastName));
        }

        return employees;
    }
}
